class Vector2D {
    constructor(x = 0, y = 0) {
        if (x instanceof Vector2D) {
            this.x = x.x;
            this.y = x.y;
        }
        else {
            this.x = x;
            this.y = y;
        }
    }
    add(x = 0, y = 0) {
        if (x instanceof Vector2D)
            return this.add(x.x, x.y);
        this.x += x;
        this.y += y;
        return this;
    }
    sub(x = 0, y = 0) {
        if (x instanceof Vector2D)
            return this.sub(x.x, x.y);
        this.x -= x;
        this.y -= y;
        return this;
    }
    scale(n) {
        this.x *= n;
        this.y *= n;
        return this;
    }
    lenSqr() {
        const { x, y } = this;
        return x * x + y * y;
    }
    len() {
        return Math.sqrt(this.lenSqr());
    }
    normalize() {
        const len = this.len();
        len !== 0 && this.scale(1 / len);
        return this;
    }
    dist(x = 0, y = 0) {
        if (x instanceof Vector2D)
            return x.copy().sub(this).len();
        return this.dist(new Vector2D(x, y));
    }
    equals(x = 0, y = 0) {
        if (x instanceof Vector2D)
            return this.equals(x.x, x.y);
        return this.x === x && this.y === y;
    }
    dot(x = 0, y = 0) {
        if (x instanceof Vector2D)
            return this.dot(x.x, x.y);
        return this.x * x + this.y * y;
    }
    lerp(x = 0, y = 0, percent = 0.5) {
        if (x instanceof Vector2D)
            return this.lerp(x.x, x.y, y || 0.5);
        this.x += percent * (x - this.x);
        this.y += percent * (y - this.y);
        return this;
    }
    nlerp(x = 0, y = 0, percent = 0.5) {
        if (!(x instanceof Vector2D))
            return this.nlerp(new Vector2D(x, y), percent);
        return this.lerp(x, y || 0.5).normalize();
    }
    copy() {
        return new Vector2D(this);
    }
}
export default Vector2D;
